"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var hoistNonReactStatic = require("hoist-non-react-statics");
var Subscription_1 = require("react-redux/lib/utils/Subscription");
var createObservable_1 = require("../utils/createObservable");
var invariant = require("invariant");
var react_1 = require("react");
var Context_1 = require("./Context");
function checkModelType(model, name, modelTypeName) {
    var instanceCon = modelTypeName[name];
    if (instanceCon == null) {
        modelTypeName[name] = model.constructor;
    }
    else {
        invariant(instanceCon === model.constructor, "Different Model should not use the same model name, Please check name: " + name);
    }
}
function storeStateUpdatesReducer(state) {
    var updateCount = state[0];
    return [updateCount + 1];
}
var initStateUpdates = function () { return [0]; };
var EMPTY_ARRAY = [];
function subscribe(modelMap, dependences, extraOptions) {
    if (dependences === void 0) { dependences = []; }
    if (extraOptions === void 0) { extraOptions = {}; }
    return function wrapWithSubscribe(WrappedComponent) {
        var Context = extraOptions.context || Context_1.default;
        var Subscribe = React.memo(function (props) {
            var contextValue = React.useContext(Context);
            var store = contextValue.store;
            var resa = store.resa;
            var modelMetaMap = react_1.useRef({});
            var modelMapChildProps = react_1.useRef({});
            var isInit = react_1.useRef(true);
            var _a = react_1.useMemo(function () {
                var forwardedRef = props.forwardedRef, wrapperProps = __rest(props, ["forwardedRef"]);
                return [forwardedRef, wrapperProps];
            }, [props]), forwardedRef = _a[0], wrapperProps = _a[1];
            var _b = react_1.useMemo(function () {
                var subscription = new Subscription_1.default(store, contextValue.subscription);
                var notifyNestedSubs = subscription.notifyNestedSubs.bind(subscription);
                return [subscription, notifyNestedSubs];
            }, [store, , contextValue]), subscription = _b[0], notifyNestedSubs = _b[1];
            var overriddenContextValue = react_1.useMemo(function () {
                return __assign({}, contextValue, { subscription: subscription });
            }, [contextValue, subscription]);
            var _c = react_1.useReducer(storeStateUpdatesReducer, EMPTY_ARRAY, initStateUpdates), previousStateUpdateResult = _c[0][0], forceComponentUpdateDispatch = _c[1];
            var updateObservable = react_1.useCallback(function () {
                var models = resa.models;
                var returnMap = {};
                Object.keys(modelMetaMap.current).map(function (key) {
                    var modelMeta = modelMetaMap.current[key];
                    var model = models[modelMeta.name];
                    var state = model.state;
                    if (modelMeta.state !== state) {
                        modelMeta.state = state;
                        modelMeta.observableModel = Object.assign({}, model, {
                            state: createObservable_1.default(state, modelMeta.depandenceMap),
                        });
                        returnMap[key] = modelMeta.observableModel;
                    }
                });
                modelMapChildProps.current = returnMap;
            }, []);
            var tryRegister = react_1.useCallback(function () {
                var models = resa.models;
                Object.keys(modelMap).map(function (key) {
                    var modelItem = modelMap[key];
                    var instance = new modelItem();
                    var namespace = instance.namespace;
                    var name = namespace === '' ? instance.name : namespace + "/" + instance.name;
                    var model = models[name];
                    if (process.env.NODE_ENV !== 'production') {
                        checkModelType(instance, name, resa.modelTypeName);
                        invariant(Object.prototype.toString.call(instance.state) === '[object Object]', 'The shape of state must be an object');
                    }
                    if (model == null) {
                        resa.register(instance);
                    }
                    var depandenceMap = {};
                    dependences.map(function (dp) {
                        depandenceMap[dp] = true;
                    });
                    modelMetaMap.current[key] = {
                        name: name,
                        depandenceMap: depandenceMap,
                        observableModel: null,
                        state: null,
                    };
                });
                updateObservable();
                isInit.current = false;
            }, []);
            if (isInit.current) {
                tryRegister();
            }
            var calculateShouldUpdate = react_1.useCallback(function () {
                var models = resa.models;
                return Object.keys(modelMetaMap.current).some(function (key) {
                    var modelMeta = modelMetaMap.current[key];
                    var model = models[modelMeta.name];
                    var state = model.state;
                    var prevState = modelMeta.state;
                    var dMap = modelMeta.depandenceMap;
                    return Object.keys(dMap).some(function (key) {
                        if (state[key] !== prevState[key]) {
                            return true;
                        }
                        return false;
                    });
                });
            }, []);
            var childPropsFromStoreUpdate = react_1.useRef(false);
            react_1.useLayoutEffect(function () {
                if (childPropsFromStoreUpdate.current) {
                    childPropsFromStoreUpdate.current = false;
                    notifyNestedSubs();
                }
            });
            react_1.useLayoutEffect(function () {
                var didUnsubscribe = false;
                var onStateChange = function () {
                    if (didUnsubscribe) {
                        return;
                    }
                    var shouldUpdate = calculateShouldUpdate();
                    if (shouldUpdate) {
                        updateObservable();
                        childPropsFromStoreUpdate.current = true;
                        forceComponentUpdateDispatch({
                            type: 'STORE_UPDATED',
                        });
                    }
                    else {
                        notifyNestedSubs();
                    }
                };
                subscription.onStateChange = onStateChange;
                subscription.trySubscribe();
                return function () {
                    didUnsubscribe = true;
                    subscription.tryUnsubscribe();
                };
            }, [store, subscription]);
            var renderChild = react_1.useMemo(function () {
                return (React.createElement(Context.Provider, { value: overriddenContextValue },
                    React.createElement(WrappedComponent, __assign({ ref: forwardedRef }, wrapperProps, modelMapChildProps.current))));
            }, [forwardedRef, wrapperProps, modelMapChildProps.current]);
            return renderChild;
        });
        Subscribe.WrappedComponent = WrappedComponent;
        var wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
        var displayName = "ResaSubscribe(" + wrappedComponentName + ")";
        Subscribe.displayName = displayName;
        if (extraOptions.forwardRef) {
            var forwarded = React.forwardRef(function forwardConnectRef(props, ref) {
                return React.createElement(Subscribe, __assign({}, props, { forwardedRef: ref }));
            });
            forwarded.displayName = displayName;
            forwarded.WrappedComponent = WrappedComponent;
            return hoistNonReactStatic(forwarded, WrappedComponent);
        }
        return hoistNonReactStatic(Subscribe, WrappedComponent);
    };
}
exports.default = subscribe;
